#include "rb_audiere.h"

// Use this typedef to make the compiler happy when
// calling rb_define_method()
typedef VALUE (ruby_method)(...);

extern "C" void ruby_audiere_free( RbTSoundAudiere *pRbTSoundAudiere ) {
  if( pRbTSoundAudiere != NULL )
	free(pRbTSoundAudiere);
}

extern "C" void ruby_audiere_mark( RbTSoundAudiere *pRbTSoundAudiere ) {
  if( pRbTSoundAudiere == NULL ) return;
  // if( !NIL_P(pRbTSoundAudiere->vData) ) rb_gc_mark( pRbTSoundAudiere->vData );
}

extern "C" VALUE ruby_audiere_new( VALUE self ) {
  RbTSoundAudiere *pRbTSoundAudiere;

  pRbTSoundAudiere = (RbTSoundAudiere *)malloc(sizeof(RbTSoundAudiere));
  if( pRbTSoundAudiere == NULL )
	rb_raise(rb_eNoMemError, "No memory left for Audiere struct");

  pRbTSoundAudiere->device = NULL;
  pRbTSoundAudiere->sound  = NULL;

  return( Data_Wrap_Struct( self, ruby_audiere_mark, ruby_audiere_free, pRbTSoundAudiere ) );
}

// extern "C" VALUE ruby_audiere_openDevice( VALUE self, VALUE rbSoundDevice ) {
extern "C" VALUE ruby_audiere_openDevice( int iNbArgs, VALUE *vArgs, VALUE self ) {
  char* xDevName = 0;
  char* xDevParameters = 0;
  
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  if( iNbArgs > 0 ) {
	if( TYPE( vArgs[0] ) == T_STRING ) {
	  xDevName = STR2CSTR( vArgs[0] );
	}
	if( iNbArgs > 1 ) {
	  if( TYPE( vArgs[1] ) == T_STRING ) {
	    xDevParameters = STR2CSTR( vArgs[1] );
	  }
	}
  }
  
  pRbTSoundAudiere->device = OpenDevice( xDevName, xDevParameters );
  if( !pRbTSoundAudiere->device ) {
	return( Qfalse );
  }

  return( Qtrue );
}

extern "C" VALUE ruby_audiere_openSound( VALUE self, VALUE rbSoundFile ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  if( !pRbTSoundAudiere->device ) {
	pRbTSoundAudiere->device = OpenDevice( "" );
	if( !pRbTSoundAudiere->device ) {
	  return( Qfalse );
	}
  }
  
  pRbTSoundAudiere->sound = OpenSound( pRbTSoundAudiere->device, STR2CSTR( rbSoundFile ) );
  if( !pRbTSoundAudiere->device ) {
	return( Qfalse );
  }

  return( Qtrue );
}

extern "C" VALUE ruby_audiere_play( VALUE self ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  pRbTSoundAudiere->sound->play( );

  return( Qnil );
}

extern "C" VALUE ruby_audiere_stop( VALUE self ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  if( pRbTSoundAudiere->sound->isPlaying( ) ) {
    pRbTSoundAudiere->sound->stop( );
  }

  return( Qnil );
}

extern "C" VALUE ruby_audiere_reset( VALUE self ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  pRbTSoundAudiere->sound->reset( );

  return( Qnil );
}

extern "C" VALUE ruby_audiere_isPlaying( VALUE self ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  if( pRbTSoundAudiere->sound->isPlaying( ) ) {
	return( Qtrue );
  }

  return( Qfalse );
}

extern "C" VALUE ruby_audiere_isSeekable( VALUE self ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  if( pRbTSoundAudiere->sound->isSeekable( ) ) {
	return( Qtrue );
  }

  return( Qfalse );
}

extern "C" VALUE ruby_audiere_getLength( VALUE self ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  return( INT2NUM( pRbTSoundAudiere->sound->getLength( ) ) );
}

extern "C" VALUE ruby_audiere_getPosition( VALUE self ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  return( INT2NUM( pRbTSoundAudiere->sound->getPosition( ) ) );
}

extern "C" VALUE ruby_audiere_setPosition( VALUE self, VALUE data ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  pRbTSoundAudiere->sound->setPosition( NUM2INT( data ) );
  return( Qnil );
}

extern "C" VALUE ruby_audiere_getVolume( VALUE self ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  return( rb_float_new( pRbTSoundAudiere->sound->getVolume( ) ) );
}

extern "C" VALUE ruby_audiere_setVolume( VALUE self, VALUE data ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  pRbTSoundAudiere->sound->setVolume( NUM2LONG( data ) );
  return( Qnil );
}

extern "C" VALUE ruby_audiere_getPan( VALUE self ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  return( rb_float_new( pRbTSoundAudiere->sound->getPan( ) ) );
}

extern "C" VALUE ruby_audiere_setPan( VALUE self, VALUE data ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  pRbTSoundAudiere->sound->setPan( NUM2LONG( data ) );
  return( Qnil );
}

extern "C" VALUE ruby_audiere_getPitchShift( VALUE self ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  return( rb_float_new( pRbTSoundAudiere->sound->getPitchShift( ) ) );
}

extern "C" VALUE ruby_audiere_setPitchShift( VALUE self, VALUE data ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  pRbTSoundAudiere->sound->setPitchShift( NUM2LONG( data ) );
  return( Qnil );
}

extern "C" VALUE ruby_audiere_getRepeat( VALUE self ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  if( pRbTSoundAudiere->sound->getRepeat( ) ) {
	return( Qtrue );
  }
  return( Qfalse );
}

extern "C" VALUE ruby_audiere_setRepeat( VALUE self, VALUE data ) {
  RbTSoundAudiere *pRbTSoundAudiere;
  Data_Get_Struct( self, RbTSoundAudiere, pRbTSoundAudiere );

  if( TYPE( data ) == T_TRUE ) {
    pRbTSoundAudiere->sound->setRepeat( true );
  } else if( TYPE( data ) == T_FALSE ) {
    pRbTSoundAudiere->sound->setRepeat( false );
  }
  return( Qnil );
}

VALUE mSound;
VALUE cAuriere;

extern "C" void Init_audiere() {
  mSound  = rb_define_module( "Sound" );
  cAuriere = rb_define_class_under( mSound, "Audiere", rb_cObject );
  
  rb_define_singleton_method( cAuriere, "new", (ruby_method*) &ruby_audiere_new, 0 );
  
  rb_define_method( cAuriere, "openDevice",    (ruby_method*) &ruby_audiere_openDevice,    -1 );
  rb_define_method( cAuriere, "openSound",     (ruby_method*) &ruby_audiere_openSound,      1 );

  rb_define_method( cAuriere, "play",          (ruby_method*) &ruby_audiere_play,           0 );
  rb_define_method( cAuriere, "stop",          (ruby_method*) &ruby_audiere_stop,           0 );
  rb_define_method( cAuriere, "reset",         (ruby_method*) &ruby_audiere_reset,          0 );

  rb_define_method( cAuriere, "isPlaying",     (ruby_method*) &ruby_audiere_isPlaying,      0 );
  rb_define_method( cAuriere, "isSeekable",    (ruby_method*) &ruby_audiere_isSeekable,     0 );
  rb_define_method( cAuriere, "getLength",     (ruby_method*) &ruby_audiere_getLength,      0 );
  
  rb_define_method( cAuriere, "getPosition",   (ruby_method*) &ruby_audiere_getPosition,    0 );
  rb_define_method( cAuriere, "setPosition",   (ruby_method*) &ruby_audiere_setPosition,    1 );
  
  rb_define_method( cAuriere, "getVolume",     (ruby_method*) &ruby_audiere_getVolume,      0 );
  rb_define_method( cAuriere, "setVolume",     (ruby_method*) &ruby_audiere_setVolume,      1 );
  
  rb_define_method( cAuriere, "getPan",        (ruby_method*) &ruby_audiere_getPan,         0 );
  rb_define_method( cAuriere, "setPan",        (ruby_method*) &ruby_audiere_setPan,         1 );
  
  rb_define_method( cAuriere, "getPitchShift", (ruby_method*) &ruby_audiere_getPitchShift,  0 );
  rb_define_method( cAuriere, "setPitchShift", (ruby_method*) &ruby_audiere_setPitchShift,  1 );
  
  rb_define_method( cAuriere, "getRepeat",     (ruby_method*) &ruby_audiere_getRepeat,      0 );
  rb_define_method( cAuriere, "setRepeat",     (ruby_method*) &ruby_audiere_setRepeat,      1 );
}
